﻿//using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

/* This script draws a chart with several up to several series of points
 * connected alines */
public class ChartHandler : MonoBehaviour {
	// References to prefabs
	public GameObject linePrefab;
	public GameObject axisNumberPrefab;
	public GameObject leyendItemPrefab;
	public GameObject labelPrefab;
	// Refences to the containers
	public Transform chartContainer;
	public Transform axesNumbersContainer;
	public Transform leyendContainer;
	// The limits in world space for a camera of size 5 aprox
	private Vector2 minWorld = new Vector2(-2,0);
	private Vector2 maxWorld = new Vector2(2, 4);
	// Widths of axes and lines in world space units
	private float axisWidth=0.1f;
	private float lineWidth=0.05f;
	// The limits in chart space
	private Vector2 minChart;
	private Vector2 maxChart;
	// The index of the series painted (i.e. the number of series painted from
	// the last time the axes are drowad. This is used for setting the color of
	// lines
	private int indexSeries;
	// Colors of the series
	private Color[] colors = { Color.red, Color.blue, Color.black, Color.green, 
		Color.magenta, Color.gray, Color.yellow, Color.cyan, Color.white};
	private Color colorAxes = Color.black;
	// Number of numbers shown in each axis, and their distance from the axis in space world
	// units
	private int numAxisNumbers = 5;
	private float distanceFromNumbersAxis = 0.3f;
	// Constants of distances in screen units
	private float distFromLabelScreen = 50;
	private float distFromLeyendScreen = 60;


	// Draw chart axes with certain limits in the chart space 
	// This function needs to be called at the beginning to set the limits
	// of the later series that will be added.
	// It also erases all the previous series drawn.
	public void DrawAxes(Vector2 minChart, Vector2 maxChart,
			float divisorX=1, float divisorY=1){
		Utils.ClearChildren (axesNumbersContainer);
		Utils.ClearChildren (chartContainer);
		Utils.ClearChildren (leyendContainer);
		indexSeries = 0;
		this.minChart = minChart;
		this.maxChart = maxChart;
		DrawAxes (divisorX, divisorY);
	}

	// Add a X label
	public void AddXLabel(string text){
		Vector2 pos = new Vector2 ((minWorld.x + maxWorld.x) / 2, minWorld.y);
		Vector2 screenPos = Camera.main.WorldToScreenPoint (pos);
		screenPos.y = screenPos.y - distFromLabelScreen;
		GameObject label = Instantiate (labelPrefab);
		label.transform.SetParent (axesNumbersContainer, false);
		label.transform.position = screenPos;
		label.GetComponent<Text> ().text = text;
	}

	// It draw the leyends from a list of series names
	public void DrawLeyends (string[] seriesNames){
		// Add the leyends
		for (int i = 0; i < seriesNames.Length; i++) {
			GameObject leyendItem = Instantiate (leyendItemPrefab);
			leyendItem.GetComponentInChildren<Text> ().text = seriesNames [i];
			leyendItem.transform.SetParent (leyendContainer, false);
			leyendItem.GetComponentInChildren<Image> ().color = colors[i];
		}
		// Place the leyend
		Vector2 pos = new Vector2((minWorld.x+maxWorld.x)/2, minWorld.y); 
			//+ new Vector2 (0, -distanceFromLeyend);
		Vector2 screenPos = Camera.main.WorldToScreenPoint (pos);
		screenPos.y -= distFromLeyendScreen;
		leyendContainer.position = screenPos;
	}

	// It draw the chart including the axes and the series. The axes
	// are drawn considering the maximum and mininmum limits of all the
	// series
	// It has some divisors to adapt scales of axes
	public void DrawAxesAndSeriesY(List<float>[] severalSeriesY, float divisorX=1, 
				float divisorY=1){
		if (severalSeriesY.Length <= 0)
			return;
		// Calculate the minimum and maximum of several  series
		float minX, maxX, minY, maxY;
		minX = 0;
		maxX = severalSeriesY[0].Count-1; // We substrat 1, to not count the zero
		minY = float.MaxValue;
		maxY = float.MinValue;
		foreach (List<float> seriesY in severalSeriesY) {
			minY = Mathf.Min (minY, Utils.Min(seriesY));
			maxY = Mathf.Max (maxY, Utils.Max(seriesY));
		}
		Vector2 minChart = new Vector2 (minX, minY);
		Vector2 maxChart = new Vector2 (maxX, maxY);
		// Draw the axes and series
		DrawAxes (minChart, maxChart, divisorX, divisorY);
		foreach(List<float> seriesY in severalSeriesY)
			DrawSeriesY (seriesY);
	}

	// It draw a series by only receiving the values of the y axis, assuming the
	// x increments one by one, as it is commong in ABS for representing
	// iterations in the x value. 
	// For using this method, one needs to invoke first "DrawAxes"
	// It has some divisors to adapt scales of axes
	public void DrawSeriesY(List<float> seriesY){
		// Check for errors
		if (seriesY.Count < 2)
			return;
		// Get the first and draw the series
		Vector2 prev = new Vector2(0,seriesY[0]);
		for(int i = 1; i<seriesY.Count; i++){
			Vector2 current = new Vector2 (i, seriesY [i]);
			DrawLineChart (prev, current);
			prev = current;
		}
		// Increase the index of series
		indexSeries++;
	}

	// Draw the axes of the chart
	private void DrawAxes(float divisorX=1, float divisorY=1){
		// Draw X axis
		Vector2 init = minWorld;
		Vector2 end = new Vector2 (maxWorld.x, minWorld.y);
		DrawLineWorld (init, end, axisWidth,true);
		DrawAxisNumbers(init, end, new Vector2(0,- distanceFromNumbersAxis), minChart.x, maxChart.x,
			divisorX);
		// Draw Y axis
		end = new Vector2(minWorld.x,maxWorld.y);
		DrawLineWorld (init, end, axisWidth,true);
		DrawAxisNumbers(init, end, new Vector2(- distanceFromNumbersAxis,0), minChart.y, maxChart.y,
			divisorY);
	}
	// Draw the all the numbers of an axis, from an init poistion to end position
	// in the world coordinates with a give offset
	private void DrawAxisNumbers(Vector2 init, Vector2 end, Vector2 offset,
			float minValue, float maxValue, float divisor = 1){
		float ratioInc = 1f / ((float)numAxisNumbers);
		float currentRatio=0;
		int prevNumber = 0;
		for (int i = 0; i < numAxisNumbers; i++) {
			currentRatio += ratioInc;
			int number = (int)((currentRatio * (maxValue - minValue) + minValue)/divisor);
			if (number > prevNumber) {
				Vector2 pos = Vector2.Lerp (init, end, currentRatio) + offset;
				DrawAxisNumber (number, pos);
			}
			prevNumber = number;
		}
	}
	// Draw an axis number on a world space position
	private void DrawAxisNumber(int number, Vector2 worldPos){
		GameObject axisNumber = Instantiate (axisNumberPrefab);
		axisNumber.GetComponent<Text>().text = "" + number;
		axisNumber.transform.SetParent (axesNumbersContainer, false);
		Vector3 screenPos = Camera.main.WorldToScreenPoint(worldPos);
		axisNumber.transform.position = screenPos;
	}


	// It draws a line from one point to another considering the chart space 
	private void DrawLineChart(Vector2 init, Vector2 end){
		DrawLineWorld (ChartToWorld (init), ChartToWorld (end), lineWidth);
	}
	// It transforms a point in the chart coordinates into the world coordinates
	private Vector2 ChartToWorld (Vector2 pointChart){
		Vector2 worldPoint = new Vector2 ();
		float ratioX = (pointChart.x - minChart.x) / (maxChart.x - minChart.x);
		worldPoint.x = minWorld.x + ratioX * (maxWorld.x - minWorld.x);
		float ratioY = (pointChart.y - minChart.y) / (maxChart.y - minChart.y);
		worldPoint.y = minWorld.y + ratioY * (maxWorld.y - minWorld.y);
		return worldPoint;
	}

	// It draws a line from one point to another considering the world space 
	private void DrawLineWorld(Vector2 init, Vector2 end, float lineWidth, bool isAxis=false)
	{
		GameObject line = Object.Instantiate(linePrefab);
		line.GetComponent<LineHandler>().PlaceLine(init, end, lineWidth);
		line.transform.SetParent(chartContainer);
		Color color= colorAxes;
		if(!isAxis && (indexSeries<colors.Length))
			color = colors[indexSeries];
		line.GetComponent<Renderer> ().material.SetColor ("_Color", color);
	}

}
