﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

/* This class handles an line that is represented with a Quad. Basically, it can be configured
 * to go from one 2D space point to another. */
public class LineHandler : MonoBehaviour
{
    // The vertices of the mesh 
    private Vector3[] vertices;
    // The base texture coordinates of the texture
    private Vector2[] uv;
    // The triangles of the mesh
    private int[] triangles;
    // Use this for initialization
    void Start()
    {
        //InitializeMeshComponents ();
        //UpdateMesh ();	
        //PlaceEdge(new Vector2(0,0), new Vector3(1,3));
    }

    /* It initializes the some default values for the mesh components */
    private void InitializeMeshComponents()
    {
        vertices = new Vector3[] {
            new Vector3 (0, 0, 0), new Vector3 (1, 0, 0), new Vector3 (0, 1, 0), new Vector3 (1, 1, 0)
        };
        uv = new Vector2[] {
            new Vector2 (0, 0), new Vector2 (1, 0), new Vector2 (0, 1), new Vector2 (1, 1)
        };
        triangles = new int[] {
            0, 3, 1,
            3, 0, 2
        };
    }

    /* This function sets the vertices of the mesh */
    private void UpdateMesh()
    {
        Mesh mesh = GetComponent<MeshFilter>().mesh;
        mesh.Clear();
        mesh.vertices = vertices;
        mesh.uv = uv;
        mesh.triangles = triangles;
        mesh.RecalculateNormals();
    }
    /* Creates a line from one point to another in the 2D space */
    public void PlaceLine(Vector2 init, Vector2 end, float lineWidth)
    {
        Vector2 edgeDirection = (end - init).normalized;
        // Vector that will be summed to init and end points the get respective right and 
        // left vertices
        Vector3 right = new Vector3(edgeDirection.y, -edgeDirection.x, 0) * (lineWidth / 2);
        Vector3 left = new Vector3(-edgeDirection.y, edgeDirection.x, 0) * (lineWidth / 2);
        // Use 3D coordinates
        Vector3 init3 = new Vector3(init.x, init.y, 0);
        Vector3 end3 = new Vector3(end.x, end.y, 0);
        // Set the vertices and update
        InitializeMeshComponents();
        vertices[0] = init3 + left;
        vertices[1] = init3 + right;
        vertices[2] = end3 + left;
        vertices[3] = end3 + right;
        UpdateMesh();
    }
    /* Creates a triangle between three points and a given color. The
     * order of positions is relevant (against clockwise)*/
    public void PlaceTriangle(Vector2 pos0, Vector2 pos1, Vector3 pos2)
    {
        // Set the vertices and update
        InitializeMeshComponents();
        vertices[0] = new Vector3(pos0.x,pos0.y);
        vertices[1] = new Vector3(pos1.x, pos1.y);
        vertices[2] = new Vector3(pos2.x, pos2.y);
        vertices[3] = new Vector3(pos2.x, pos2.y);
        UpdateMesh();
    }
}