﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

/* This class is assigned to the starplot label button to make appear and disappear de text
 * labels */
public class StarplotLabelButtonHandler : MonoBehaviour {
	// This button contains the text that is displayed
	public GameObject textLabelButton; 
	// Comparative size of the text in comparison with the image
	private float widthText=2.6f;
	private float heightText=0.8f;
	// threshold for considering the sin the angle up or down 
	private float sinThreshold = 0.8f;


	// It initializes some values according to the prefab
	/*
	void Awake(){
		RectTransform rect = (RectTransform) textLabelButton.transform;
		widthText =  rect.anchorMax.x - rect.anchorMin.x;
		heightText = rect.anchorMax.y - rect.anchorMin.y;
		Debug.Log ("widthText " + widthText+ " heightText " + heightText);
	}*/



	// This function just makes appear and disappear the text label
	public void PressImageButton(){
		// Close all the other text displays for opening it
		if (!textLabelButton.activeInHierarchy) {
			StarplotLabelButtonHandler[] labelButtons = transform.parent.GetComponentsInChildren
			               < StarplotLabelButtonHandler> ();
			foreach (StarplotLabelButtonHandler handler in labelButtons)
				handler.HideText ();
		}
		// Change the state of the text button
		textLabelButton.SetActive (!textLabelButton.activeInHierarchy);
	}
	// It hides the text 
	private void HideText(){
		textLabelButton.SetActive (false);
	}

	// It sets the text of the label, adding the space for the "X" button for closing
	public void SetTextLabel(string text){
		textLabelButton.GetComponentInChildren<Text> ().text = text+ " X";
	}

	// It adjusts the right position of the text label given certain angle of the starplot
	public void AdjustTextPosition(float angle){
		// Find the relative position of the text
		float x=0, y=0;
		// If the sin of the agle surpasses certain absolute amount then the text
		// will be up or down regarding its sign
		if (Mathf.Abs (Mathf.Sin (angle)) >= sinThreshold) {
			y = Mathf.Sign (Mathf.Sin (angle));
			x = Mathf.Cos (angle);
			// Otherwise the it will be positioned left or right the opposite sign
		} else {
			x = -Mathf.Sign (Mathf.Cos (angle));
		}
		// Scale with the width and height all the anchor positions
		RectTransform rect = (RectTransform) textLabelButton.transform;
		x = 0.5f + x * 0.5f + x * widthText / 2;
		y = 0.5f + y * 0.5f + y * heightText / 2;
		rect.anchorMin = new Vector2(x-widthText/2f, y-heightText/2f);
		rect.anchorMax = new Vector2(x+widthText/2f, y + heightText/2f);
		//Debug.Log("Adjusting text position :"+x+","+y+". Anchors: "+rect.anchorMin+","+
		//	rect.anchorMax);
	}
}
