﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System;

/* This agent represents the manager of resource/service of a smart city */
public class ManagerAgent : Agent{
	// Queues for storing the users that hav requested a service
	List<Request>[] queues; 
	//  Counter of the resquests of all the queues
	int countRequest;
	// Constructor 
	public ManagerAgent(Simulation sim):base(sim){
		queues = new List<Request>[(int)Params.maxNormPrior+1]; // add the zero level
		for (int i = 0; i < queues.Length; i++)
			queues [i] = new List<Request> ();
		countRequest = 0;
	}
	// It attends the user agents in the queue following the ration among queues
	// until it achieves the maximum number of possible requests to be attended
	public override void Live(){
		int countAttended = 0;
		// It attends the number of services it can if there are requests pending
		while (countAttended < Params.numServices && countRequest>0){
			// In each queue:
			for (int normPrior = 0; normPrior < queues.Length && 
					countAttended < Params.numServices; normPrior++) {
				// It attends as much requests as established unther ti reachs the services
				for (int j = 0; j < Params.numServicesPriorities [normPrior]&&
						countAttended < Params.numServices; j++) {
					// If not empty attend the request
					if (queues [normPrior].Count > 0) {
						AttendRequest (normPrior);
						countAttended++;
					}
				}
			}
		}
	}
	// It adds a request for a giever user agent received by parameter
	// and a priority, and  in all the lower priority levels
	public void AddRequest(Request request){
		for(int i=request.normPrior; i<queues.Length;i++)
			queues [i].Add (request);
		countRequest++;
	}

	// It attends a request of a given norm. priority level if any.
	// It removes the request from all the queues
	private void AttendRequest(int normPrior){
		if (queues [normPrior].Count > 0) {
			Request request = queues [normPrior] [0];
			request.timeEnd=sim.GetIteration();
			for(int i=0; i<queues.Length;i++)
				queues [i].Remove (request);
			request.userAgent.ReceiveService (request);
			countRequest--;
		}
	}
}
