﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System;

// This agent observes the controller agent and keeps track of the evolution
public class ObserverAgent : StochasticUserAgent {
	// The evloution of the measures
	public List<float>[] evolutions;
	// Number of metrics and their names
	public string[] namesMetrics; 
	// The accumulated times for each priority level
	float[] accumTimes;
	// The number of attended requests for each prioirty level
	float[] countTimes;

	// It initializes the data structures in the constructor
	public ObserverAgent(Simulation sim): base(sim) {
		// It always asks for service to measure
		probsAsk[0]=1;
		probsAsk[1]=1;
		// It asks enough time each priority to observe, but without exceeding
		probsPriorities[0]=0.2f;
		probsPriorities [1] = 0.2f;
		// It initiate sthe names of metrics
		namesMetrics= new string[(int)Params.maxPriority+1];
		for (int i = 0; i < namesMetrics.Length; i++) {
			namesMetrics[i] = "Priority " + i+ " (s)"; 
		}
		// It initiates the envolutions
		evolutions = new List<float>[namesMetrics.Length];
		for (int i = 0; i < namesMetrics.Length; i++)
			evolutions [i] = new List<float> ();
		accumTimes = new float[(int)Params.maxPriority + 1];
		countTimes = new float[(int)Params.maxPriority + 1];
	}

	// It records the waiting times
	public override void ReceiveService(Request request){
		float time = (float) (request.timeEnd - request.timeBegin);
		accumTimes [(int) request.priority] += time;
		countTimes [(int) request.priority]++;
	}
		
	// Add the initial values, once the observer agent is with all the other 
	// agengs but before running the simulation, for getting the values
	// at the beginning. In this way, one-iteration simulation outputs some chart.
	// and the initial values are represented.
	public void AddInitialValues(){
		// Clear data in the iteration zero
		ClearData ();
		// Last pose/poster
		RecordMetricValues ();
	}

	// It records the evolution. If the iteration zero, it clears
	// first the data
	public override void Live () {
		base.Live ();
		RecordMetricValues();
	}
		
	// Record the exiting averages indicated by metrics
	private void RecordMetricValues(){
		float[] results = CalculateResultsMetrics ();
		for (int i = 0; i < results.Length; i++)
			evolutions [i].Add(results [i]);
	}

	// Calculate a list of the results
	public float[] CalculateResultsMetrics(){
		float[] results = new float[namesMetrics.Length];
		for (int i = 0; i < results.Length; i++) {
			if(countTimes[i]>0)
				results [i] = accumTimes[i]/countTimes[i];
		}
		return results;
	}
	// Clear all the data of the evolution
	private void ClearData(){
		foreach (List<float> evolution in evolutions)
			evolution.Clear ();
		for (int i = 0; i < countTimes.Length; i++) {
			accumTimes [i] = 0;
			countTimes [i] = 0;
		}
	}

	// It saves the evolution in into a file if activate this functionality
	// in the experimentation
	public void SaveEvolutionIntoFile(){
		if (!Experimentation.saveLogsEvolution)
			return;
		string str = "";
		foreach (string metricName in namesMetrics)
			str += metricName + "\t";
		str += "\n";
		for (int i= 0; i< evolutions[0].Count;i++) {
			for(int j=0; j<evolutions.Length;j++)
				str += evolutions[j][i] + "\t";
			str += "\n";
		}
		Experimentation.SaveResults (str, Experimentation.pathLogsEvol,false);
	}


}
	
