﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using System;

/* This class is in charge of presenting the results to the user. */
public class ResultsPresenter : MonoBehaviour {
	// Handler of the starplot and the chart
	public StarplotHandler starplotHandler;
	public ChartHandler chartHandler;
	// References to the sprites of the starplot
	public Sprite[] sprites;
	// Singleton instance
	private static ResultsPresenter instance;

	// Singleton
	public static ResultsPresenter Instance(){
		if (instance == null) 
			instance = (ResultsPresenter) GameObject.FindObjectOfType (typeof(ResultsPresenter));
		return instance;
	}

	// It shows the evolutions of a simulation, considering all the series
	// considering the different kinds of observer agents
	public void ShowEvolutions(ObserverAgent observerAgent){
		MenuManager.Instance().GoTo("ChartEvolution");
		chartHandler.DrawAxesAndSeriesY(observerAgent.evolutions);
		chartHandler.DrawLeyends (observerAgent.namesMetrics);
		chartHandler.AddXLabel ("Simulated time (s)");
	}
		
	// It shows all the results in the starplots considering the different kinds
	//  of oabserver agents
	public void ShowStarplotResults (ObserverAgent observerAgent) {
		MenuManager.Instance().GoTo("StarplotMainPrefab");
		float[] results = observerAgent.CalculateResultsMetrics ();
		// Goes to the specifc screen and empty it
		starplotHandler.RemoveAllElements ();
		// Create the starplot
		starplotHandler.CreateStarplotFromValues (results);
		// Create the labels
		string[] texts = new string[observerAgent.namesMetrics.Length];
		for (int i = 0; i < texts.Length; i++)
			texts [i] = observerAgent.namesMetrics[i]+"\n" + 
				results[i].ToString("0.0") + "s";
		starplotHandler.CreateLabelsFromImagesTexts(GetSprites(),texts);
	}

	// Load the sprites only the first time and returns it always
	private Sprite[] GetSprites(){
		return sprites;
	}
}