﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

/* This agent records the requests of their user agent. It it detects a 
 * misuse of the priority mechanism, it assigns the lowest priority. 
 * Otherwisi, it assigns the priority that ir requests. */
public class SmartPriorityAgent : PriorityAgent {
	// Accumulation of the requested priorities
	float accumPrior;
	// Number of requests
	float numRequests;
	// The threshold of priorities  that is substracted from the maximum
	float substractThreshold =0.9f;
	// Threshold calculated from the substraction
	float threshold;

	// Constructor
	public SmartPriorityAgent(Simulation sim, ManagerAgent managerAgent):
	base(sim,managerAgent){
		accumPrior = 0;
		numRequests = 0;
		threshold = Params.maxPriority - substractThreshold;
	}
	// It detects misuse according to threshold
	protected override int Normalize(float priority){
		// It updates the information 
		accumPrior+=priority;
		numRequests++;
		// The first time, it trusts the agent as it has no history
		if (numRequests <= 1)
			return base.Normalize (priority);
		// All the other times, it calculates the average and decices
		float avg = accumPrior/numRequests;
		if (avg >= threshold)
			return base.Normalize (priority);
		else
			return base.Normalize(Params.maxPriority);
	}
}
