﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System;

// This agent selects the priorities in a nondeterministic way given certain probabilities.
// The subclasses may reassign these probabilities in the constructor.
// It represents a user that is honest as the frequency of request with high priorities is low
public class StochasticUserAgent : UserAgent {
	// The probabilities in taking the decision of each priority level, except the last
	// one that would be one minus the others
	protected float[] probsPriorities={0.1f,0.1f};
	// It determines the number of iteration in which it is long term
	protected int longTermDuration = 10;
	// Probablities for asking service respevely in the short time and the long term
	protected float[] probsAsk = {0.8f, 0.3f};

	public StochasticUserAgent(Simulation sim):base(sim){
	}

	// In the short term and long term, it uses different probabilities
	protected override bool DecideIfAskService(){
		int isLongTerm = Convert.ToInt32 (sim.GetIteration () >= longTermDuration);
		float rand = UnityEngine.Random.Range (0f, 1f);
		float prob = probsAsk [isLongTerm];
		bool result = (rand <= prob);
		return result;
	}

	// Selects the priority each time it requires a service
	protected override float SelectPriority(){
		float rand = UnityEngine.Random.Range (0f, 1f);
		float accumProb = 0;
		// All the cases except the last one
		for (int priority = 0; priority < probsPriorities.Length; priority++) {
			accumProb += probsPriorities [priority];
			if (rand <= accumProb)
				return priority;
		}
		// The last case
		return Params.maxPriority;
	}

}
