﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

/* Represents a citizen, company or organization that uses a service/resource
 * of the smart city. Some methods has very basic behaviors, as these
 * are intended to be overwritten by the subclasses.
 * */
public class UserAgent : Agent {
	// Its priority agent for contacting the service
	private PriorityAgent priorityAgent;

	// Constructor of the class
	public UserAgent(Simulation sim):base(sim){
	}
	// It connects to a priority agent 
	public void SetPriorityAgent(PriorityAgent priorityAgent){
		this.priorityAgent = priorityAgent;
	}

	// It perodiacallty asks for a service with a given priority
	public override void Live ()
	{
		if(DecideIfAskService()){
			AskService ();
		}
	}

	// It decides whether to ask for a service
	protected virtual bool DecideIfAskService(){
		return true;
	}

	// The agent simulates the selection of a priority
	protected virtual float SelectPriority(){
		return Params.maxPriority;
	}

	// It asks a service through its priority agent
	protected void AskService(){
		Request request = new Request ();
		request.priority=SelectPriority (); 
		request.timeBegin = sim.GetIteration ();
		request.userAgent = this;
		priorityAgent.AskService (request);
	}

	// It is informed that it receives the service
	// for a certain request. This for example is useful for the observer
	// agent that reports the statistics.
	// In this case, it receives the service, and just does nothing.
	public virtual void ReceiveService(Request request){
		//Debug.Log ("Receiving service " + request);
	}
}
